/* *************** */
/* Fitxer sonar.c  */	
/* *************** */

#include "saphira.h"

// Constants

#define MIN_D 10	// Es defineix com a buffer mnim 10 (dret)
#define MIN_E 10	// Es defineix com a buffer mnim 10 (esquerra)
#define MIN_F 5		// Es defineix com a buffer mnim 10 (frontal)

#define TOL_MIN 0.001	// Es defineix el 0.0 com qualsevol n<TOL_MIN
#define PI 3.1416

// Definici Funcions Internes

float c_dist (float x, float y);	// Retorna el mdul del vector (x,y)
float c_angle (float x, float y);	// Retorna l'angle (deg.) del vector (x,y)
int dintre_q (float x, float y, float x0, float y0, float x1, float y1);
void Normalitzar (float *x0, float *y0, float *x1, float *y1);

// Variables globals

float b_sonar_x[CBUF_LEN];
float b_sonar_y[CBUF_LEN];
int b_sonar;

float b_sonar_x_d[CBUF_LEN];
float b_sonar_y_d[CBUF_LEN];
int b_sonar_d;

float b_sonar_x_e[CBUF_LEN];
float b_sonar_y_e[CBUF_LEN];
int b_sonar_e;

float b_sonar_x_f[CBUF_LEN];
float b_sonar_y_f[CBUF_LEN];
int b_sonar_f;

// Funcions Internes

float c_dist (float x, float y)
{
	return (float)sqrt(pow(x,2) + pow(y,2));
}

float c_angle (float x, float y)
{
	float angle = 0.0;
	
	if (x<TOL_MIN)
	{
		if (y>0.0) angle = 90.0;
		if (y<0.0) angle = -90.0;
	}

	if (y<TOL_MIN)
	{
		if (x>0.0) angle = 0.0;
		if (x<0.0) angle = 180.0;
	}

	if ((x>TOL_MIN)&&(y>TOL_MIN))
	{
		angle = (float)((180/PI)*(atan(y/x)));
	}

	if ((x>TOL_MIN)&&(y<TOL_MIN))
	{
		angle = (float)((180/PI)*(-atan(-y/x)));
	}

	if ((x<TOL_MIN)&&(y>TOL_MIN))
	{
		angle = (float)(180 - ((180/PI)*atan(y/-x)));
	}
	
	if ((x<TOL_MIN)&&(y<TOL_MIN))
	{
		angle = (float)(-180 + ((180/PI)*atan(y/x)));
	}

	return angle;
}

int dintre_q (float x, float y, float x0, float y0, float x1, float y1)
{
	return ((x>x0)&&(x<x1)&&(y>y0)&&(y<y1));
}

void Normalitzar (float *x0, float *y0, float *x1, float *y1)
{
	float aux;

	if (*x0>*x1)
	{
		aux = *x1;
		*x1 = *x0;
		*x0 = aux;
	}

	if (*y0>*y1)
	{
		aux = *y1;
		*y1 = *y0;
		*y0 = aux;
	}

	if (((*x0)==(*x1))||((*y0)==(*y1))) 
		sfSMessage("Quadre buit. No poden haver elements interiors");
}

// Funcions s_actualitzar

EXPORT void s_actualitzar (void)
{
	int limit, i;

	b_sonar = 0;

	limit = sr_buf->limit;
	i = 0;

	while (i<limit)
	{
		if (sr_buf->valid[i])
		{
			b_sonar_x[b_sonar] = sr_buf->xbuf[i];
			b_sonar_y[b_sonar] = sr_buf->ybuf[i];
			b_sonar++;
		}

		i++;
	}

	limit = sl_buf->limit;
	i = 0;

	while (i<limit)
	{
		if (sl_buf->valid[i])
		{
			b_sonar_x[b_sonar] = sl_buf->xbuf[i];
			b_sonar_y[b_sonar] = sl_buf->ybuf[i];
			b_sonar++;
		}

		i++;
	}

	limit = sraw_buf->limit;
	i = 0;

	while (i<limit)
	{
		if (sraw_buf->valid[i])
		{
			b_sonar_x[b_sonar] = sraw_buf->xbuf[i];
			b_sonar_y[b_sonar] = sraw_buf->ybuf[i];
			b_sonar++;
		}

		i++;
	}
}

EXPORT void s_actualitzar_d (void)
{
	int limit, i;

	b_sonar_d = 0;
	limit = sr_buf->limit;
	i = 0;

	while (i<limit)
	{
		if (sr_buf->valid[i])
		{
			b_sonar_x_d[b_sonar_d] = sr_buf->xbuf[i];
			b_sonar_y_d[b_sonar_d] = sr_buf->ybuf[i];
			b_sonar_d++;
		}

		i++;
	}
}

EXPORT void s_actualitzar_e (void)
{
	int limit, i;

	b_sonar_e = 0;
	limit = sl_buf->limit;
	i = 0;

	while (i<limit)
	{
		if (sl_buf->valid[i])
		{
			b_sonar_x_e[b_sonar_e] = sl_buf->xbuf[i];
			b_sonar_y_e[b_sonar_e] = sl_buf->ybuf[i];
			b_sonar_e++;
		}

		i++;
	}
}

EXPORT void s_actualitzar_f (void)
{
	int limit, i;

	b_sonar_f = 0;
	limit = sraw_buf->limit;
	i = 0;

	while (i<limit)
	{
		if (sraw_buf->valid[i])
		{
			b_sonar_x_f[b_sonar_f] = sraw_buf->xbuf[i];
			b_sonar_y_f[b_sonar_f] = sraw_buf->ybuf[i];
			b_sonar_f++;
		}

		i++;
	}
}

// Funcions s_maxim_x

EXPORT int s_maxim (void)
{
	return sr_buf->limit + sl_buf->limit + sraw_buf->limit;
}

EXPORT int s_maxim_d (void)
{
	return sr_buf->limit;
}

EXPORT int s_maxim_e (void)
{
	return sl_buf->limit;
}

EXPORT int s_maxim_f (void)
{
	return sraw_buf->limit;
}

// Funcions s_posar_maxim_x

EXPORT void s_posar_maxim_l (int maxim)
{
	if ((maxim<=CBUF_LEN)&&(maxim>=MIN_D)) sfSetSideBuffer(maxim);
	if (maxim<MIN_D) sfSetSideBuffer(MIN_D);
	if (maxim>CBUF_LEN) sfSetSideBuffer(CBUF_LEN);
}

EXPORT void s_posar_maxim_f (int maxim)
{
	if ((maxim<=CBUF_LEN)&&(maxim>=MIN_F)) sfSetFrontBuffer(maxim);
	if (maxim<MIN_F) sfSetFrontBuffer(MIN_F);
	if (maxim>CBUF_LEN) sfSetFrontBuffer(CBUF_LEN);
}

//Funcions s_elements_x

EXPORT int s_elements (void)
{
	return b_sonar;
}

EXPORT int s_elements_d (void)
{
	return b_sonar_d;
}

EXPORT int s_elements_e (void)
{
	return b_sonar_e;
}

EXPORT int s_elements_f (void)
{
	return b_sonar_f;
}

// Funcions s_x_x, s_y_x

EXPORT float s_x (int i)
{
	if ((i<=b_sonar)&&(i>0)) return b_sonar_x[i-1];
	else
	{	
		sfSMessage("Error d'accs al buffer. ndex incorrecte");
		return 0.0;
	}
}

EXPORT float s_y (int i)
{
	if ((i<=b_sonar)&&(i>0)) return b_sonar_y[i-1];
	else
	{	
		sfSMessage("Error d'accs al buffer. ndex incorrecte");
		return 0.0;
	}
}

EXPORT float s_x_d (int i)
{
	if ((i<=b_sonar_d)&&(i>0)) return b_sonar_x_d[i-1];
	else
	{	
		sfSMessage("Error d'accs al buffer. ndex incorrecte");
		return 0.0;
	}
}

EXPORT float s_y_d (int i)
{
	if ((i<=b_sonar_d)&&(i>0)) return b_sonar_y_d[i-1];
	else
	{	
		sfSMessage("Error d'accs al buffer. ndex incorrecte");
		return 0.0;
	}
}

EXPORT float s_x_e (int i)
{
	if ((i<=b_sonar_e)&&(i>0)) return b_sonar_x_e[i-1];
	else
	{	
		sfSMessage("Error d'accs al buffer. ndex incorrecte");
		return 0.0;
	}
}

EXPORT float s_y_e (int i)
{
	if ((i<=b_sonar_e)&&(i>0)) return b_sonar_y_e[i-1];
	else
	{	
		sfSMessage("Error d'accs al buffer. ndex incorrecte");
		return 0.0;
	}
}

EXPORT float s_x_f (int i)
{
	if ((i<=b_sonar_f)&&(i>0)) return b_sonar_x_f[i-1];
	else
	{	
		sfSMessage("Error d'accs al buffer. ndex incorrecte");
		return 0.0;
	}
}

EXPORT float s_y_f (int i)
{
	if ((i<=b_sonar_f)&&(i>0)) return b_sonar_y_f[i-1];
	else
	{	
		sfSMessage("Error d'accs al buffer. ndex incorrecte");
		return 0.0;
	}
}

// Funcions s_dist_x

EXPORT float s_dist (int i)
{
	double aux = 0.0;

	if ((i<=b_sonar)&&(i>0))
	{
		aux = c_dist(b_sonar_x[i-1],b_sonar_y[i-1]);
	}
	else sfSMessage("Error d'accs al buffer. ndex incorrecte");
	
	return (float)aux;
}

EXPORT float s_dist_d (int i)
{
	double aux = 0.0;

	if ((i<=b_sonar_d)&&(i>0))
	{
		aux = c_dist(b_sonar_x_d[i-1],b_sonar_y_d[i-1]);
	}
	else sfSMessage("Error d'accs al buffer. ndex incorrecte");
	
	return (float)aux;
}

EXPORT float s_dist_e (int i)
{
	double aux = 0.0;

	if ((i<=b_sonar_e)&&(i>0))
	{
		aux = c_dist(b_sonar_x_e[i-1],b_sonar_y_e[i-1]);
	}
	else sfSMessage("Error d'accs al buffer. ndex incorrecte");
	
	return (float)aux;
}

EXPORT float s_dist_f (int i)
{
	double aux = 0.0;

	if ((i<=b_sonar_f)&&(i>0))
	{
		aux = c_dist(b_sonar_x_f[i-1],b_sonar_y_f[i-1]);
	}
	else sfSMessage("Error d'accs al buffer. ndex incorrecte");
	
	return (float)aux;
}

// Funcions s_angle_x

EXPORT float s_angle (int i)
{
	double aux = 0.0;

	if ((i<=b_sonar)&&(i>0))
	{
		aux = c_angle(b_sonar_x[i-1],b_sonar_y[i-1]);
	}
	else sfSMessage("Error d'accs al buffer. ndex incorrecte");
	
	return (float)aux;
}

EXPORT float s_angle_d (int i)
{
	double aux = 0.0;

	if ((i<=b_sonar_d)&&(i>0))
	{
		aux = c_angle(b_sonar_x_d[i-1],b_sonar_y_d[i-1]);
	}
	else sfSMessage("Error d'accs al buffer. ndex incorrecte");
	
	return (float)aux;
}

EXPORT float s_angle_e (int i)
{
	double aux = 0.0;

	if ((i<=b_sonar_e)&&(i>0))
	{
		aux = c_angle(b_sonar_x_e[i-1],b_sonar_y_e[i-1]);
	}
	else sfSMessage("Error d'accs al buffer. ndex incorrecte");
	
	return (float)aux;
}

EXPORT float s_angle_f (int i)
{
	double aux = 0.0;

	if ((i<=b_sonar_f)&&(i>0))
	{
		aux = c_angle(b_sonar_x_f[i-1],b_sonar_y_f[i-1]);
	}
	else sfSMessage("Error d'accs al buffer. ndex incorrecte");
	
	return (float)aux;
}

// Funcions s_total_quadre_x

EXPORT int s_total_quadre (float x0, float y0, float x1, float y1)
{
	int num,i;

	// Normalitzem

	Normalitzar (&x0,&y0,&x1,&y1);
	
	// Comprovem cada element

	i = 0;
	num = 0;

	while (i<b_sonar)
	{
		if (dintre_q(b_sonar_x[i],b_sonar_y[i],x0,y0,x1,y1)) num++;

		i++;
	}
	
	return num;
}

EXPORT int s_total_quadre_d (float x0, float y0, float x1, float y1)
{
	int num,i;

	// Normalitzem

	Normalitzar (&x0,&y0,&x1,&y1);
	
	// Comprovem cada element

	i = 0;
	num = 0;

	while (i<b_sonar_d)
	{
		if (dintre_q(b_sonar_x_d[i],b_sonar_y_d[i],x0,y0,x1,y1)) num++;

		i++;
	}
	
	return num;
}

EXPORT int s_total_quadre_e (float x0, float y0, float x1, float y1)
{
	int num,i;

	// Normalitzem

	Normalitzar (&x0,&y0,&x1,&y1);
	
	// Comprovem cada element

	i = 0;
	num = 0;

	while (i<b_sonar_e)
	{
		if (dintre_q(b_sonar_x_e[i],b_sonar_y_e[i],x0,y0,x1,y1)) num++;

		i++;
	}
	
	return num;
}

EXPORT int s_total_quadre_f (float x0, float y0, float x1, float y1)
{
	int num,i;

	// Normalitzem

	Normalitzar (&x0,&y0,&x1,&y1);
	
	// Comprovem cada element

	i = 0;
	num = 0;

	while (i<b_sonar_f)
	{
		if (dintre_q(b_sonar_x_f[i],b_sonar_y_f[i],x0,y0,x1,y1)) num++;

		i++;
	}
	
	return num;
}

// Funcions s_dist_min_x

EXPORT int s_dist_min (void)
{
	int i,num;

	num = 0;

	if (b_sonar == 0) sfSMessage ("Buffer Buit");
	else num = 1;
	
	i = 1;

	while (i<b_sonar)
	{
		if (c_dist(b_sonar_x[i],b_sonar_y[i])<c_dist(b_sonar_x[num-1],b_sonar_y[num-1]))
			num = i + 1;
		
		i++;
	}

	return num;
}

EXPORT int s_dist_min_d (void)
{
	int i,num;

	num = 0;

	if (b_sonar_d == 0) sfSMessage ("Buffer Buit");
	else num = 1;
	
	i = 1;

	while (i<b_sonar_d)
	{
		if (c_dist(b_sonar_x_d[i],b_sonar_y_d[i])<c_dist(b_sonar_x_d[num-1],b_sonar_y_d[num-1]))
			num = i + 1;
		
		i++;
	}

	return num;
}

EXPORT int s_dist_min_e (void)
{
	int i,num;

	num = 0;

	if (b_sonar_e == 0) sfSMessage ("Buffer Buit");
	else num = 1;
	
	i = 1;

	while (i<b_sonar_e)
	{
		if (c_dist(b_sonar_x_e[i],b_sonar_y_e[i])<c_dist(b_sonar_x_e[num-1],b_sonar_y_e[num-1]))
			num = i + 1;
		
		i++;
	}

	return num;
}

EXPORT int s_dist_min_f (void)
{
	int i,num;

	num = 0;

	if (b_sonar_f == 0) sfSMessage ("Buffer Buit");
	else num = 1;
	
	i = 1;

	while (i<b_sonar_f)
	{
		if (c_dist(b_sonar_x_f[i],b_sonar_y_f[i])<c_dist(b_sonar_x_f[num-1],b_sonar_y_f[num-1]))
			num = i + 1;
		
		i++;
	}

	return num;
}

// Funcions s_dist_max_x

EXPORT int s_dist_max (void)
{
	int i,num;

	num = 0;

	if (b_sonar == 0) sfSMessage ("Buffer Buit");
	else num = 1;
	
	i = 1;

	while (i<b_sonar)
	{
		if (c_dist(b_sonar_x[i],b_sonar_y[i])>c_dist(b_sonar_x[num-1],b_sonar_y[num-1]))
			num = i + 1;
		
		i++;
	}

	return num;
}

EXPORT int s_dist_max_d (void)
{
	int i,num;

	num = 0;

	if (b_sonar_d == 0) sfSMessage ("Buffer Buit");
	else num = 1;
	
	i = 1;

	while (i<b_sonar_d)
	{
		if (c_dist(b_sonar_x_d[i],b_sonar_y_d[i])>c_dist(b_sonar_x_d[num-1],b_sonar_y_d[num-1]))
			num = i + 1;
		
		i++;
	}

	return num;
}

EXPORT int s_dist_max_e (void)
{
	int i,num;

	num = 0;

	if (b_sonar_e == 0) sfSMessage ("Buffer Buit");
	else num = 1;
	
	i = 1;

	while (i<b_sonar_e)
	{
		if (c_dist(b_sonar_x_e[i],b_sonar_y_e[i])>c_dist(b_sonar_x_e[num-1],b_sonar_y_e[num-1]))
			num = i + 1;
		
		i++;
	}

	return num;
}

EXPORT int s_dist_max_f (void)
{
	int i,num;

	num = 0;

	if (b_sonar_f == 0) sfSMessage ("Buffer Buit");
	else num = 1;
	
	i = 1;

	while (i<b_sonar_f)
	{
		if (c_dist(b_sonar_x_f[i],b_sonar_y_f[i])>c_dist(b_sonar_x_f[num-1],b_sonar_y_f[num-1]))
			num = i + 1;
		
		i++;
	}

	return num;
}

// Funcions s_elements_bola_x

EXPORT int s_total_cercle (float radi)
{
	int num,i;

	// Comprovem cada element

	i = 0;
	num = 0;

	while (i<b_sonar)
	{
		if (c_dist(b_sonar_x[i],b_sonar_y[i])<radi) num++;

		i++;
	}
	
	return num;
}

EXPORT int s_total_cercle_d (float radi)
{
	int num,i;

	// Comprovem cada element

	i = 0;
	num = 0;

	while (i<b_sonar_d)
	{
		if (c_dist(b_sonar_x_d[i],b_sonar_y_d[i])<radi) num++;

		i++;
	}
	
	return num;
}

EXPORT int s_total_cercle_e (float radi)
{
	int num,i;

	// Comprovem cada element

	i = 0;
	num = 0;

	while (i<b_sonar_e)
	{
		if (c_dist(b_sonar_x_e[i],b_sonar_y_e[i])<radi) num++;

		i++;
	}
	
	return num;
}

EXPORT int s_total_cercle_f (float radi)
{
	int num,i;

	// Comprovem cada element

	i = 0;
	num = 0;

	while (i<b_sonar_f)
	{
		if (c_dist(b_sonar_x_f[i],b_sonar_y_f[i])<radi) num++;

		i++;
	}
	
	return num;
}
