/* ************************ */
/* FITXER sfReachPosition.c	*/
/* ************************ */

#include "saphira.h"
#include "sfReachPosition.h"
#include "beh.c"

// ------------------------------
// Definici de funcions internes
// ------------------------------

void ActivarComportaments(void)
{
	if (!inactive)
	{
		if (en_funcionament) sfrStart();
		else sfrStop();
	}
}

// --------------------
// Codi de les funcions
// --------------------

void sfrInactive(void)
{
	sfRemoveTask("assolir_pos");
	sfRemoveTask("assolir_angle");
	sfRemoveTask("sfrStop");

	inactive = 1;
}

void sfrStart(void)
{
	sfStartBehavior(sfAnarAPos,"assolir_pos", 0, p_priority, 0,
		velocitat, p_dist_tol);
	sfStartBehavior(sfTurnToPosition,"assolir_angle", 0, p_priority, 0);
	sfStartBehavior(sfStop,"sfrStop", 0, p_priority, 1);

	en_funcionament = 1;
	inactive = 0;
}

void sfrStop(void)
{
	sfStartBehavior(sfAnarAPos,"assolir_pos", 0, p_priority, 1,
		velocitat, p_dist_tol);
	sfStartBehavior(sfTurnToPosition,"assolir_angle", 0, p_priority, 1);
	sfStartBehavior(sfStop,"sfrStop", 0, p_priority, 0);

	en_funcionament = 0;
	inactive = 0;
}

void sfrSetPosition(float x, float y, float angle)
{
	llCambiarPunt(punts,0,x,y,angle);

	x = x+L_VECTOR*(float)cos(angle*2*PI/360);
	y = y+L_VECTOR*(float)sin(angle*2*PI/360);

	llCambiarPunt(punts,1,x,y,angle);

	ActivarComportaments();
}

void sfrSetGlobalPosition(float x, float y, float angle)
{
	llCambiarPuntGlobal(punts,0,x,y,angle);

	x = x+L_VECTOR*(float)cos(angle*2*PI/360);
	y = y+L_VECTOR*(float)sin(angle*2*PI/360);

	llCambiarPuntGlobal(punts,1,x,y,angle);

	ActivarComportaments();
}

void sfrSetVelocity(float v)
{
	velocitat = v;

	// Per tal de fer el canvi de velocitat inmediatament 
	// es tornen a iniciar els comportaments

	ActivarComportaments();
}

float sfrGetXPosition(void)
{
	return llObtenirXPunt(punts,0);
}

float sfrGetYPosition(void)
{
	return llObtenirYPunt(punts,0);
}

float sfrGetAnglePosition(void)
{
	return llObtenirAnglePunt(punts,0);
}

float sfrGetXGlobalPosition(void)
{
	return llObtenirXGlobalPunt(punts,0);
}

float sfrGetYGlobalPosition(void)
{
	return llObtenirYGlobalPunt(punts,0);
}

float sfrGetGlobalAnglePosition(void)
{
	return llObtenirAngleGlobalPunt(punts,0);
}

float sfrGetVelocity(void)
{
	return velocitat;
}

int sfrIsPositionReach(void)
{
	return ((sfGetTaskState("assolir_pos")==sfSUCCESS)&&  
		(abs(llAnglePunt(punts,1))<p_angle_tol));
}

int sfrIsPointReach(void)
{
	return ((sfGetTaskState("assolir_pos")==sfSUCCESS));
}

float sfrGetDistancePosition(void)
{
	return llDistanciaPunt(punts,0);
}

// ------------------------------------
// Codi dels parmetres de funcionament
// ------------------------------------

int sfrGetPriority(void)
{
	return p_priority;
}

float sfrGetDistTol(void)
{
	return p_dist_tol;
}

float sfrGetAngleTol(void)
{
	return p_angle_tol;
}

void sfrSetPriority(int priority)
{
	p_priority = priority;

	ActivarComportaments();
}

void sfrSetDistTol(float tol)
{
	p_dist_tol = tol;

	ActivarComportaments();
}

void sfrSetAngleTol(float tol)
{
	p_angle_tol = tol;

	ActivarComportaments();
}

// -----------------------------------------
// Fitxers de crrega i descrrega de la dll
// -----------------------------------------

EXPORT void
sfLoadInit(void)	// Fitxer que s'avalua al carregar-se la dll
{
	// Missatge d'inici

	sfSMessage("Iniciant crrega sfReachPosition.dll");

	// Crrega de les funcions a Colbert

	sfAddEvalFn("sfrSetPosition", sfrSetPosition, sfVOID, 3, sfFLOAT, sfFLOAT, sfFLOAT );
	sfAddEvalFn("sfrSetGlobalPosition", sfrSetGlobalPosition, sfVOID, 3, sfFLOAT, sfFLOAT, sfFLOAT );
	sfAddEvalFn("sfrSetVelocity", sfrSetVelocity, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfrGetXPosition", sfrGetXPosition, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrGetYPosition", sfrGetYPosition, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrGetAnglePosition", sfrGetAnglePosition, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrGetXGlobalPosition", sfrGetXGlobalPosition, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrGetYGlobalPosition", sfrGetYGlobalPosition, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrGetGlobalAnglePosition", sfrGetGlobalAnglePosition, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrGetVelocity", sfrGetVelocity, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrIsPositionReach", sfrIsPositionReach, sfINT, 0, sfVOID);
	sfAddEvalFn("sfrIsPointReach", sfrIsPointReach, sfINT, 0, sfVOID);
	sfAddEvalFn("sfrGetDistancePosition", sfrGetDistancePosition, sfFLOAT, 0, sfVOID);

	sfAddEvalFn("sfrGetPriority", sfrGetPriority, sfINT, 0, sfVOID);
	sfAddEvalFn("sfrSetPriority", sfrSetPriority, sfVOID, 1, sfINT);
	sfAddEvalFn("sfrGetDistTol", sfrGetDistTol, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrSetDistTol", sfrSetDistTol, sfVOID, 1, sfFLOAT);
	sfAddEvalFn("sfrGetAngleTol", sfrGetAngleTol, sfFLOAT, 0, sfVOID);
	sfAddEvalFn("sfrSetAngleTol", sfrSetAngleTol, sfVOID, 1, sfFLOAT);

	sfAddEvalFn("sfrStart", sfrStart, sfVOID, 0, sfVOID);
	sfAddEvalFn("sfrStop", sfrStop, sfVOID, 0, sfVOID);
	sfAddEvalFn("sfrInactive", sfrInactive, sfVOID, 0, sfVOID);

	// sfRegisterSchema("sfTurnToPosition",sfBEHAVIOR,sfTurnToPosition);

	// Inicialitzar variables per defecte

	punts = llCrearLlistaPunts();

	llAfegirPunt(punts,0,0,0);
	llAfegirPunt(punts,L_VECTOR,0,0);

	velocitat = 100.0;
	
	// Missatge final

	sfSMessage("Crrega de sfReachPosition.dll finalitzada");
}

EXPORT void
sfLoadExit(void)	// Fitxer que s'avalua al descarregar-se la dll
{
	sfrInactive();

	// Treiem els punts de la llista

	llDestruirLlistaPunts(punts);

	// Missatge de descrrega

	sfSMessage("Descrrega de reachPosition.dll efectuada");
}
