/* ******************* */
/* Fitxer sfTracking.h */
/* ******************* */

// -------------------
// Inclusi de fitxers
// -------------------

#include "sfUtils.h"

// -----------------------------------------------
// Funci principal (ha d'estar en un bloc update)
// -----------------------------------------------

void c_calcul_contorn(void);

// -----------------------------
// Funcions que agafen les dades
// -----------------------------

void c_agafar_dades(void);

// ----------------------------------------------
// Funcions que realitzen un filtrat de les dades
// ----------------------------------------------

// Crida als filtres

void c_aplicar_filtres(void);

void c_filtre_angle(LlistaPunts l, float alfa_1, float alfa_2, float max_correcte);
void c_filtre_focus(LlistaPunts l, float dist_focus);
void c_filtre_recta(LlistaPunts l);

// Funcions filtres

int correcte_filtre_angle(float x, float y, float alfa_1, float alfa_2, float max_correcte);

// --------------------------------------------------------------
// Funcions que calculen el control (x_reg, y_reg, angle_control)
// --------------------------------------------------------------

void calcular_cami(void);
void calcular_angle_cami(void);

void c_calcul_control(float x, float y, float *angle, float *dist);

void c_angle_control_y(float b0, float b1, float *angle);
void c_angle_control_x(float b0, float b1, float *angle);
void c_distancia_control(float x, float y, float *dist);

float c_calcularXReg(void);
float c_calcularYReg(void);

// -------------------------------------------------------------
// Funci de monitoritzaci (veure vars internes des de Colbert)
// -------------------------------------------------------------

void c_monitoritzar(int opcions);
void c_visualitzar();

// ------------------------------
// Definici de funcions internes
// ------------------------------

// Funcions geomtriques: mdul (valor absolut) i distancia

float modul(float x);						
float distancia(float x, float y);			

// Funcions que realitzen una regresi (i clculs associats)

void c_calcul_mitjanes(LlistaPunts l, float *x_mitja, float *y_mitja); 
void c_calcul_variances(LlistaPunts l, float *cov_xy, float *sx_2, float *sy_2);
void c_calcul_recta_regressio(LlistaPunts l, float *x0, float *y0, float *vx, float *vy, float *r);
void c_calcul_r(LlistaPunts l, float x0, float y0, float vx, float vy, float *r);
void c_calcul_projeccio(float x1, float y1, float x0, float y0, float vx, float vy, float *xp, float *yp);

// ----------------------
// Definici de constants
// ----------------------

// Constants numriques

#define PI 3.1416		// Nombre PI
#define TOL_MIN 0.001	// Qualsevol valor menor ser tractat com 0.0

// -----------------------
// Definici de parmetres
// -----------------------

// Parmetres del contorn

float distancia_a_mantenir;		// Var que cont la distancia a mantenir
float velocitat_a_mantenir;		// Var que cont la velocitat a mantenir

int seguir_dreta = 1;			// Seguiment d'objectes a la dreta = 1, a l'esquerra = 0
int monitoritzacio = 0;			// Monitoritzaci a aplicar
int visualitzacio = 7;			// Visualitzaci dels punts a aplicar

// Parametres de control

float angle_sens = (float)0.1;		// Sensibilitat a l'angle
float dist_sens = (float)0.1;		// Sensibilitat a la distancia
float parada_sens = 0.0;			// Maxim parada possible quan angle o distancia incorrecte
float p_dist_radi_cami = 1000.0;	// Distancia d'intersecci amb recta (Calcul cami)
float p_focus_control = 100.0;		// Distancia del focus control (seguiment de contorn)

// Parmetres dels filtres

int filtre_angle_activat = 1;		// Filtre angle activat = 1, desactivat = 0
int filtre_focus_activat = 1;		// Filtre focus activat = 1, desactivat = 0
int filtre_recta_activat = 1;		// Filtre recta activat = 1, desactivat = 0

float p_alfa_1 = 15.0;				// Angle recta 1 per ser correctes (Filtre recta)
float p_alfa_2 = 45.0;				// Angle recta 2 per ser correctes (Filtre recta)
float p_max_correcte = 3.0;			// Factor multiplicatiu per distancia (Filtre recta)
float p_dist_focus = 500.0;			// Distancia mxima del focus (Filtre focus)

// Parametres emergncia

float angle_control_emergencia = 25.0;
float r_min_emergencia = (float)0.2;
float r_max_emergencia = (float)0.2;
int metode_calcul_r = 1;
float const_dist_emergencia = 1.0;

// ------------------------------------------------------
// Definici de funcions per fixar i veure els parametres
// ------------------------------------------------------

float sfTGetDistanciaAMantenir(void);
void sfTSetDistanciaAMantenir(float d);

float sfTGetVelocitatAMantenir(void);
void sfTSetVelocitatAMantenir(float v);

int sfTIsSeguimentDreta(void);
void sfTSetSeguimentDreta(void);
int sfTIsSeguimentEsquerra(void);
void sfTSetSeguimentEsquerra(void);

int sfTGetMonitoritzacio(void);
void sfTSetMonitoritzacio(int m);
int sfTGetVisualitzacio(void);
void sfTSetVisualitzacio(int v);

float sfTGetAngleSensibilitat(void);
void sfTSetAngleSensibilitat(float s);
float sfTGetDistanciaSensibilitat(void);
void sfTSetDistanciaSensibilitat(float s);
float sfTGetParadaSensibilitat(void);
void sfTSetParadaSensibilitat(float s);
float sfTGetDistanciaRadiControl(void);
void sfTSetDistanciaRadiControl(float s);
float sfTGetDistanciaFocusControl(void);
void sfTSetDistanciaFocusControl(float d);

int sfTIsFiltreAngleActivat(void);
int sfTIsFiltreFocusActivat(void);
int sfTIsFiltreRectaActivat(void);

void sfTSetFiltreAngleActivat(void);
void sfTSetFiltreFocusActivat(void);
void sfTSetFiltreRectaActivat(void);

void sfTSetFiltreAngleDesactivat(void);
void sfTSetFiltreFocusDesactivat(void);
void sfTSetFiltreRectaDesactivat(void);

float sfTGetPAlfa1(void);
void sfTSetPAlfa1(float p);
float sfTGetPAlfa2(void);
void sfTSetPAlfa2(float p);
float sfTGetPMaxCorrecte(void);
void sfTSetPMaxCorrecte(float p);
float sfTGetPDistFocus(void);
void sfTSetPDistFocus(float p);
float sfTGetPDistPunts(void);
void sfTSetPDistPunts(float p);

float sfTGetAngleControlEmergencia(void);
float sfTGetRMinEmergencia(void);
float sfTGetRMaxEmergencia(void);
void sfTSetAngleControlEmergencia(float v);
void sfTSetREmergencia(float min, float max);
int sfTGetMetodeCalculR(void);
void sfTSetMetodeCalculR(int mr);
float sfTGetConstDistEmergencia();
void sfTSetConstDistEmergencia(float c);

// ----------------------
// Definici de variables
// ----------------------

// Llista de dades (x,y)

LlistaPunts cami;				// Llista de punts que marca el cam a seguir
LlistaPunts llista;				// Llista de punts amb les dades originals (i filtrades)
LlistaPunts llista_emergencia;	// Llista de punts per seguiment d'objectes o per emergncies

// Variables de control

float angle_control;		// Variable per controlar l'angle des de lgica difusa
float distancia_control;	// Variable per controlar la distancia des de lgica difusa

float angle_cami_control;	// Variable que determina l'angle del punt del cam a seguir

float r_reg; 

