#ifndef _SFUTILS_H
#define _SFUTILS_H

#include "saphira.h"

// --------------------------
// Definici del tipus vector
// --------------------------

typedef struct 
{
    float *telem;               /* Taula d'elements */
    int dim;                    /* Dimensio */
} Vector;

// --------------------------
// Definici del tipus llista
// --------------------------

typedef struct datos      
{
  point *punt;
  struct datos *siguiente;
} telemento;

typedef struct
{
	telemento **punts;
	int *numpunts;
}LlistaPunts;

// -------------------------------
// Definici de funcions de Vector
// -------------------------------

EXPORT Vector CrearVector(int n);
EXPORT void DestruirVector(Vector v);
EXPORT void InicialitzarVector(Vector v);

EXPORT int DimVector(Vector v);
EXPORT float ConsVector(Vector v, int i);
EXPORT void AssigVector(Vector v, int i, float f);
EXPORT void EscriureVector(Vector v);

EXPORT LlistaPunts ObtenirLlistaVectors(Vector x, Vector y, Vector angle);  
EXPORT LlistaPunts ObtenirLlistaVectorsGlobals(Vector x, Vector y, Vector angle);  

// -------------------------------
// Definici de funcions de Llista
// -------------------------------

EXPORT LlistaPunts llCrearLlistaPunts(void);
EXPORT void llDestruirLlistaPunts(LlistaPunts l);

EXPORT int llNumPunts(LlistaPunts l);

EXPORT void llAfegirPunt(LlistaPunts l, float x, float y, float angle);
EXPORT void llAfegirPuntGlobal(LlistaPunts l, float x, float y, float angle);

EXPORT void llInsertarPunt(LlistaPunts l, float x, float y, float angle, int pos);
EXPORT void llInsertarPuntGlobal(LlistaPunts l, float x, float y, float angle, int pos);

EXPORT void llEliminarPunt(LlistaPunts l, int pos);
EXPORT void llEliminarPrimerPunt(LlistaPunts l);
EXPORT void llEliminarUltimPunt(LlistaPunts l);
EXPORT void llEliminarTotsPunts(LlistaPunts l);

EXPORT void llCambiarPunt(LlistaPunts l, int pos, float x, float y, float angle);
EXPORT void llCambiarPuntGlobal(LlistaPunts l, int pos, float x, float y, float angle);

EXPORT float llObtenirXPunt(LlistaPunts l, int pos);
EXPORT float llObtenirYPunt(LlistaPunts l, int pos);
EXPORT float llObtenirAnglePunt(LlistaPunts l, int pos);

EXPORT float llObtenirXGlobalPunt(LlistaPunts l, int pos);
EXPORT float llObtenirYGlobalPunt(LlistaPunts l, int pos);
EXPORT float llObtenirAngleGlobalPunt(LlistaPunts l, int pos);

EXPORT Vector llObtenirVectorXLlista(LlistaPunts l);
EXPORT Vector llObtenirVectorYLlista(LlistaPunts l);
EXPORT Vector llObtenirVectorAngleLlista(LlistaPunts l);

EXPORT Vector llObtenirVectorXGlobalLlista(LlistaPunts l);
EXPORT Vector llObtenirVectorYGlobalLlista(LlistaPunts l);
EXPORT Vector llObtenirVectorAngleGlobalLlista(LlistaPunts l);

EXPORT void llEscriureLlistaPunts(LlistaPunts l);
EXPORT void llEscriureLlistaPuntsGlobals(LlistaPunts l);

// ---------------------
// Funcions geomtriques
// ---------------------

EXPORT LlistaPunts CalcularSpline(LlistaPunts l, Vector knots, int k, int num_punts);
EXPORT LlistaPunts CalcularStandardSpline(LlistaPunts l, int k, int num_punts);

EXPORT LlistaPunts CalcularBezier(LlistaPunts l, int k, int num_punts);

EXPORT int MinimaDistanciaLlistaPunts(LlistaPunts l);
EXPORT int MaximaDistanciaLlistaPunts(LlistaPunts l);

EXPORT float llDistanciaPunt(LlistaPunts l, int pos);
EXPORT float llAnglePunt(LlistaPunts l, int pos);

EXPORT float llLongitudLlista(LlistaPunts l);
EXPORT float llLongitudLlistaInterval(LlistaPunts l, int inici, int fi); 
// Si es vol inici 0 es pot posar -1, si es vol final ltim posar -1)

#endif